/**
 * JASMINe
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: NotificationUpdateToNotification.java 8306 2011-06-14 07:22:15Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.transformers;

import java.io.IOException;
import java.util.Map;

import javax.naming.NamingException;

import org.mule.config.i18n.MessageFactory;
import org.mule.transformers.AbstractTransformer;
import org.mule.umo.UMOException;
import org.mule.umo.lifecycle.Startable;
import org.mule.umo.transformer.TransformerException;
import org.ow2.jasmine.event.beans.JasmineEventNotification;
import org.ow2.jasmine.event.beans.JasmineEventNotificationUpdate;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventNotificationSLBRemote;
import org.ow2.jasmine.monitoring.eventswitch.connectors.EJB3Connector;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Transformers that convert a {@link JasmineEventNotificationUpdate} to a
 * {@link JasmineEventNotification}, taking values that identify the
 * notification from the update message, and retrieve the updated notification
 * from the database.
 *
 * @author Guillaume Renault
 */
public class NotificationUpdateToNotification extends AbstractTransformer {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(this.getClass());

    /**
     * Configuration to get the EJB 3. Must be static, as a new Transformer is
     * Instantiated by Mule each time needed, and the configuration, set using
     * Spring, is not kept in any other ways ...
     */
    protected static Map<String, String> configuration = null;

    /**
     * EJB3 wrapper.
     */
    private EJB3NotificationWrapper wrapperEJB3 = null;

    /**
     * Registers the transformer on Mule.
     *
     * @see org.mule.transformers.AbstractTransformer
     */
    public NotificationUpdateToNotification() {
        super();
        this.registerSourceType(JasmineEventNotificationUpdate.class);
        this.setReturnClass(JasmineEventNotification.class);
    }

    /**
     * Transformer. Takes the notification from the database, and return the
     * updated {@link JasmineEventNotification}.
     */
    @Override
    protected Object doTransform(final Object paramObject, final String paramString) throws TransformerException {

        JasmineEventNotification result = null;

        try {

            if (paramObject instanceof JasmineEventNotificationUpdate) {
                JasmineEventNotificationUpdate updateData = (JasmineEventNotificationUpdate) paramObject;
                if (null == wrapperEJB3) {
                    try {
                        this.start();
                    } catch (UMOException e) {
                        throw new TransformerException(e.getI18nMessage(), e);
                    }
                }
                JasmineEventNotificationSLBRemote remote = wrapperEJB3.getNotificationRemoteBean();

                if (null != updateData.getId()) {
                    result = remote.getEventNotification(updateData.getId());
                }
            }

        } catch (NamingException e) {
            logger.warn("EJBs are not ready ... the message is not persisted");
        } catch (IOException e) {
            logger.warn("EJBs are not ready ... the message is not persisted");
        }
        return result;

    }

    /**
     * @param configuration the configuration to set
     */
    public final void setConfiguration(final Map<String, String> configuration) {
        NotificationUpdateToNotification.configuration = configuration;
    }

    /**
     * Connect the transformer to the Stateless.
     *
     * @see Startable#start()
     */
    public void start() throws UMOException {
        wrapperEJB3 = new EJB3NotificationWrapper(NotificationUpdateToNotification.configuration);
        try {
            wrapperEJB3.connect();
        } catch (Exception e) {
            throw new TransformerException(MessageFactory
                .createStaticMessage("Error while connecting to the Notification EJB3"), e);
        }

    }

    /**
     * Disconnect the transformer from the Stateless.
     *
     * @see Startable#stop()
     */
    public void stop() throws UMOException {
        wrapperEJB3.disconnect();
    }

    /**
     * Wrapper of the EJB3 handling the {@link JasmineEventNotification} in the
     * database.
     *
     * @author Guillaume Renault
     */
    private class EJB3NotificationWrapper extends EJB3Connector<JasmineEventNotificationSLBRemote> {
        public EJB3NotificationWrapper(final Map<String, String> conf) {
            this.setConfiguration(conf);
        }

        public JasmineEventNotificationSLBRemote getNotificationRemoteBean() throws NamingException, IOException {
            return this.getBean();
        }
    }
}
