/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: NotificationUpdateToNotificationUpdateWithID.java 8267 2011-06-07 16:08:42Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.transformers;

import org.mule.config.i18n.MessageFactory;
import org.mule.transformers.AbstractTransformer;
import org.mule.umo.UMOException;
import org.mule.umo.transformer.TransformerException;
import org.ow2.jasmine.event.beans.JasmineEventNotification;
import org.ow2.jasmine.event.beans.JasmineEventNotificationUpdate;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventNotificationSLBRemote;
import org.ow2.jasmine.monitoring.eventswitch.connectors.EJB3Connector;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import javax.naming.NamingException;
import java.io.IOException;
import java.util.Map;

/**
 * Transformers that consolidate a {@link org.ow2.jasmine.event.beans.JasmineEventNotificationUpdate} adding the ID of
 * {@link org.ow2.jasmine.event.beans.JasmineEventNotification} to update
 *
 * @author Julien Legrand
 */
public class NotificationUpdateToNotificationUpdateWithID extends AbstractTransformer {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(this.getClass());

    /**
     * Configuration to get the EJB 3. Must be static, as a new Transformer is
     * Instantiated by Mule each time needed, and the configuration, set using
     * Spring, is not kept in any other ways ...
     */
    protected static Map<String, String> configuration = null;

    /**
     * EJB3 wrapper.
     */
    private EJB3NotificationWrapper wrapperEJB3 = null;

    /**
     * Max number of try when get ID fails
     */
    private int maxRetry = 0;

    /**
     * Delay between 2 tries
     */
    private int delay = 0;

    /**
     * Registers the transformer on Mule.
     *
     * @see org.mule.transformers.AbstractTransformer
     */
    public NotificationUpdateToNotificationUpdateWithID() {
        super();
        this.registerSourceType(JasmineEventNotificationUpdate.class);
        this.setReturnClass(JasmineEventNotificationUpdate.class);
    }

    /**
     * Transformer. Takes the notification from the database, and return the
     * updated {@link org.ow2.jasmine.event.beans.JasmineEventNotification}.
     */
    @Override
    protected Object doTransform(final Object paramObject, final String paramString) throws TransformerException {

        JasmineEventNotificationUpdate result = null;

        if (paramObject instanceof JasmineEventNotificationUpdate) {
            try {

                result = (JasmineEventNotificationUpdate) paramObject;

                if (null == wrapperEJB3) {
                    try {
                        this.start();
                    } catch (UMOException e) {
                        throw new TransformerException(e.getI18nMessage(), e);
                    }
                }

                JasmineEventNotificationSLBRemote remote = wrapperEJB3.getNotificationRemoteBean();
                JasmineEventNotification notif = remote.getEventNotification(result.getSearchFilters());
                int i = 0;
                while (null == notif && i < maxRetry) {
                    try {
                        Thread.sleep(delay);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    notif = remote.getEventNotification(result.getSearchFilters());
                    i++;
                }

                if (null != notif) {
                    result.setId(notif.getId());
                }

            } catch (NamingException e) {
                logger.warn("EJBs are not ready ... the message is not persisted");
            } catch (IOException e) {
                logger.warn("EJBs are not ready ... the message is not persisted");
            }
        }
        
        return result;
    }

    /**
     * @param configuration the configuration to set
     */
    public final void setConfiguration(final Map<String, String> configuration) {
        NotificationUpdateToNotificationUpdateWithID.configuration = configuration;

        if (configuration.containsKey("max-retry")) {
            maxRetry = new Integer(configuration.get("max-retry"));
        }

        if (configuration.containsKey("delay-in-second")) {
            delay = new Integer(configuration.get("delay-in-second")) * 1000;
        }
    }

    /**
     * Connect the transformer to the Stateless.
     *
     * @see org.mule.umo.lifecycle.Startable#start()
     */
    public void start() throws UMOException {
        wrapperEJB3 = new EJB3NotificationWrapper(NotificationUpdateToNotificationUpdateWithID.configuration);
        try {
            wrapperEJB3.connect();
        } catch (Exception e) {
            throw new TransformerException(MessageFactory
                .createStaticMessage("Error while connecting to the Notification EJB3"), e);
        }

    }

    /**
     * Disconnect the transformer from the Stateless.
     *
     * @see org.mule.umo.lifecycle.Startable#stop()
     */
    public void stop() throws UMOException {
        wrapperEJB3.disconnect();
    }

    /**
     * Wrapper of the EJB3 handling the {@link org.ow2.jasmine.event.beans.JasmineEventNotification} in the
     * database.
     *
     * @author Guillaume Renault
     */
    private class EJB3NotificationWrapper extends EJB3Connector<JasmineEventNotificationSLBRemote> {
        public EJB3NotificationWrapper(final Map<String, String> conf) {
            this.setConfiguration(conf);
        }

        public JasmineEventNotificationSLBRemote getNotificationRemoteBean() throws NamingException, IOException {
            return this.getBean();
        }
    }
}
